from ._fftconvolve import fftconvolve2d
from ..structures.weights import gauss

from math import ceil

def fftgauss(img, sigma, theta=0, pad_type=None, **kwargs):
    '''
    Smooths the input image with a gaussian kernel. Uses the fft method and allows
    specifying a custom pad type with **kwargs from numpy.pad documentation. Smoothing
    a color image will smooth each color channel individually.

    Parameters
    ----------
    img : (M,N) or (M,N,3) array
        An image to be smoothed
    sigma : tuple or float
        Tuple defining the standard deviation of the gaussian in x and y directions.
        A single value will assign the same value to x and y st. devs..
    theta : float, optional
        The rotation of the gaussian in radians.
    pad_type : string, optional
        The padding type to be used. For additional information see numpy.pad .
        Defaults to constant.
    kwargs : varies
        See numpy.pad . Defaults to constant_values=0.

    Returns
    -------
    smoothed_image : ndarray
        A smoothed image. Keeps same shape and same number of color channels.

    Notes
    -----
    There are many gaussian smoothing functions. This one is unique because it
    automatically handles color images. It also allows defining very unique
    gaussian kernels with strain and orientation.
    '''
    s = img.shape[:2]
    K = gauss(sigma, theta, size=s)

    if isinstance(sigma, list):
        r = 2*ceil(max(sigma))
    else:
        r = sigma

    return fftconvolve2d(img, K, r=r, pad_type=pad_type, centered=True, **kwargs)



import numpy as np
import scipy.ndimage as ndi
from ..structures.shapes import circle

def smooth_binary(S, r=1, add_cond=None, rem_cond=None):
    '''
    Smooths a segmented image, where the input is a binary segmentation.

    Parameters
    ----------
    S : ndarray of booleans
        A binary image.
    add_cond : float, optional
        Parameter that specifies the percentage of neighboring pixels
        that must be in the segmentation for that pixel to be converted to
        the segmentation.
    rem_cond : float, optional
        Parameter that specifies the percentage of neighboring pixels
        outside the segmentation that cause a pixel to be removed from
        the segmentation.
    r : numeric, optional
        The radius of the kernel used to define the neighboring pixels.

    Returns
    -------
    smooth_segmentation : ndarray of booleans
        A smoothed version of the segmented image.

    Notes
    -----
    By default, pixels are only added to a segmentation. Alternatively, pixels
    can be removed and not added. Adding and removing does not make sense with
    this algorithm since a pixel used to calculate an add condition could be
    removed later resulting in a non-smooth segmentation. This method can be
    thought of as a generalization of binary erosion and dilation.
    '''

    K = circle(r, dtype=np.uint8)
    S = S.astype(np.uint8)

    if add_cond:
        add_cond_met = ndi.convolve(S, K, mode='reflect') > add_cond * np.sum(K)
        return np.logical_or(S, add_cond_met)

    elif rem_cond:
        rem_cond_met = ndi.convolve(1-S, K, mode='reflect') > rem_cond * np.sum(K)
        return np.logical_and(S, np.logical_not(rem_cond_met))

    else:
        raise ValueError('Either add_cond or rem_cond must be specified.')



import os
from os import listdir
from PIL import Image
from random import shuffle
from scipy.io import loadmat, savemat
import numpy as np


class dirload():
    '''
    Reads a sorted list of files from the specified directory if those files are of type tif, tiff, jpg, jpeg, png, or PNG. Can specify the type if desired.

    Will then allow accessing these files via slicing.

    Parameters
    ----------
    folderpath : string
    order : string, optional
        Any of {sorted or random}.
    mode : string, optional
        Any of {train, test, predict, read}.
    train_T : function, optional
        Function that transforms the training input.
    test_T : function, optional
        Function that transforms the test input.
    predict_T : function, optional
        Function that transforms the predict input.
    read_T : function, optional
        Function that transforms the read input.
    ftype : str, optional
        Specifies the type of the file that is includes in files
    prefix : str, optional
        Specifies the prefix of the file that is included in files

    Attributes
    ----------
    path : string
        Folder path.
    mode : string
        Specifies how the reader class loads data.
    files : list
        List of all files in folder.
    T : function
        Function that transforms loaded data.
    ftype : str, optional
        Specifies the type of the file that is includes in files
    prefix : str, optional
        Specifies the prefix of the file that is included in files
    '''


    def __init__(self, folderpath, order='sorted', mode='predict', train_T=lambda x:x, test_T=lambda x:x, predict_T=lambda x:x, read_T=lambda x:x, ftype=None, prefix=None):
        assert order in {'sorted', 'random'}
        assert mode in {'train', 'test', 'predict', 'read'}
        self.T = {'train':train_T, 'test':test_T, 'predict':predict_T, 'read':read_T}

        self.path = folderpath
        self.mode = mode

        self.ftype = ftype
        self.prefix = prefix

        self.files = listdir(folderpath)
        self.files = list(i for i in self.files if path.isfile(path.join(folderpath, i)))

        if ftype:
            self.files = list(i for i in self.files if i[-len(ftype):]==ftype)
        else:
            self.files = list(i for i in self.files if i.split('.')[-1] in {'jpg', 'jpeg', 'png', 'PNG', 'tif', 'tiff'})

        if prefix:
            self.files = list(i for i in self.files if i[:len(self.prefix)]==self.prefix)

        if order is 'sorted':
            self.files = sorted(self.files)
        elif order is 'random':
            self.files = shuffle(self.files)


    def __len__(self):
        return len(self.files)


    def __str__(self):
        return 'Is %s a directory? '%self.path + str(path.isdir(self.path))


    def print_all_files_(self):
        '''
        Function that prints all files in the folder.

        Prints
        ------
        all_files : str
            Each file separated by `,` in files.
        '''
        return '\n' + ',    '.join(self.files)


    def change_mode(self, mode):
        '''
        Function that changes how the reader loads data.

        Parameters
        ----------
        mode : str
        '''
        assert mode in {'train', 'predict', 'read'}
        self.mode = mode


    def update_list(self):
        '''
        If any files were added to or removed from the folder, then this
        function should be run to update files that will be loaded.

        Updates
        -------
        files : str
            All files in the folder that meet the prefix and ftype rules.
        '''
        self.files = listdir(folderpath)
        self.files = list(i for i in self.files if path.isfile(i))
        if ftype:
            self.files = list(i for i in self.files if i[-len(self.ftype):]==self.ftype)
        if prefix:
            self.files = list(i for i in self.files if i[:len(self.prefix)]==self.prefix)
        self.files = sorted(self.files)


    def getpath(self, idx):
        return path.join(self.path, self.files[idx])


    def getname(self, idx):
        return ''.join(self.files[idx].split('.')[:-1])


    def __getitem__(self, idx):
        return self.T[self.mode](Image.open(path.join(self.path, self.files[idx])))


class rwformat():
    '''
    Reads and writes from a folder by combining the prefix, ftype,
    and passed idx value. Can set width of the idx with idx_len.

    Parameters
    ----------
    folderpath : string
    idx_len : int, optional
        How long the numeric value of the name should be.
    ftype : str, optional
        Specifies the type of the file that is includes in files
    prefix : str, optional
        Specifies the prefix of the file that is included in files

    Attributes
    ----------
    path : string
        Folder path.
    idx_len : int, optional
        How long the numeric value of the name should be.
    ftype : str, optional
        Specifies the type of the file that is includes in files
    pre : str, optional
        Specifies the prefix of the file that is included in files
    '''
    def __init__(self, folderpath, idx_len=None, prefix='', ftype='.png'):

        self.path = folderpath
        self.idx_len = idx_len
        self.pre = prefix
        self.ftype = ftype

    def __len__(self):
        '''
        Finds how many files have been written to the folder or are
        currently in the folder
        '''
        files = listdir(self.path)
        files = list(i for i in files if path.isfile(path.join(self.path, i)))
        if self.ftype:
            files = list(i for i in files if i[-len(self.ftype):]==self.ftype)
        if self.pre:
            files = list(i for i in files if i[:len(self.pre)]==self.pre)
        return len(files)

    def __str__(self):
        return 'Is %s a directory? '%self.path + str(path.isdir(self.path))

    def print_all_files_(self):
        '''
        Function that prints all files in the folder.

        Prints
        ------
        all_files : str
            Each file separated by `,` in files.
        '''
        return '\n' + ',    '.join(self.files)

    def __getitem__(self, idx):
        if self.idx_len:
            num = '{number:0{width}d}'.format(width=self.idx_len, number=idx)
        else:
            num = str(idx)

        path = os.path.join(self.path, self.pre + num + self.ftype)

        if self.ftype in {'.jpeg', '.jpg', '.png', '.PNG', '.tif', '.tiff'}:
            return Image.open(path)
        elif self.ftype == '.mat':
            return loadmat(path)
        elif self.ftype == '.npy':
            return np.load(path)
        else:
            raise ValueError('Unsupported filetype (ftype)')

    def __setitem__(self, idx, val):
        if self.idx_len:
            num = '{number:0{width}d}'.format(width=self.idx_len, number=idx)
        else:
            num = str(idx)

        if self.ftype in {'.jpeg', '.jpg', '.png', '.PNG', '.tif'}:
            val.save(self.path+self.pre+num+self.ftype)
        elif self.ftype == '.mat':
            savemat(self.path+self.pre+num+self.ftype, val)
        elif self.ftype == '.npy':
            np.save(self.path+self.pre+num+self.ftype, val)



from .fftedges import local_avg
from .fftgaussian import fftgauss
from .morphological import smooth_binary
from ._fftconvolve import fftconvolve2d

__all__ = ['local_avg',
           'fftgauss',
           'smooth_binary',
           'fftconvolve2d']



# Importing general libraries
import numpy as np
import scipy.ndimage as ndi
import pandas as pd
from PIL import Image
import matplotlib.pyplot as plt
import json
import os

# Importing specific functions
from skimage.morphology import remove_small_objects, remove_small_holes, watershed
from skimage.feature import corner_peaks
from skimage.segmentation import relabel_sequential
from skimage.measure import regionprops
from skimage.color import label2rgb

# Importing ImageMKS functions
from ..filters import fftgauss, local_avg, smooth_binary
from ..structures import donut
from ..masking import maskfourier
from ..visualization import make_boundary_image
from ..rw import dirload



def _gen_marks(label_img):
    props = regionprops(label_img)
    markers = np.zeros(label_img.shape)
    label_val = 1

    for i in props:
        x, y = i.centroid
        x = int(round(x))
        y = int(round(y))
        markers[x,y] = label_val
        label_val += 1

    return markers


def default_parameters(cell_type):
    '''
    Generates a dictionary of default paramaters.

    Parameters
    ----------
    cell_type : string
        Either muscle_progenitor or bone_stem. More support coming soon.

    Returns
    -------
    params : dictionary
        Params defines smooth_size, peak_min_dist, intensity_exp,
        short_threshold_r, long_threshold_r, cytoskeleton_threshold_r,
        and min_size_objects.
    '''
    if cell_type is 'muscle_progenitor':
        params = {
            'smooth_size': 3,
            'peak_min_dist': 10,
            'intensity_exp': 2,
            'short_threshold_r': 50,
            'long_threshold_r': 600,
            'cytoskeleton_threshold_r': 200,
            'min_size_objects': 300,
            }

        return params

    elif cell_type is 'bone_stem':

        params = {
            'smooth_size': 3,
            'peak_min_dist': 10,
            'intensity_exp': 3,
            'short_threshold_r': 100,
            'long_threshold_r': 800,
            'cytoskeleton_threshold_r': 200,
            'min_size_objects': 1100,
            }

        return params

    else:
        print('Sorry this cell type is not yet supported.')


def all_measurements():
    measures = ['Cell_Index', 'Nuc_Area_um2', 'Nuc_Perimeter_um', 'Nuc_Area_Factor',
                'Nuc_Major_L_um', 'Nuc_Minor_L_um', 'Nuc_eccentricity', 'Nuc_orientation',
                'Nucleus_eq_diameter_um', 'Cyto_Area_um2', 'Cyto_Perimeter_um', 'Cyto_Area_Factor',
                'Cyto_orientation', 'Cyto_Major_L_um', 'Cyto_Minor_L_um']

    return measures


def segment_fluor_cells(N, C, pixel_scale, smooth_size, peak_min_dist, intensity_exp,
                        short_threshold_r, long_threshold_r, cytoskeleton_threshold_r,
                        min_size_objects):
    '''
    Segments fluorescent cells.

    Parameters
    ----------
    N : (M,N,3) numpy array
        A color image of nuclei with size (M,N,3)
    C : (M,N,3) numpy array
        A color image of the cytoskeleton with same size as the nucleus image (M,N,3).
    pixel_scale : float
        Real measurement in micrometers of a pixel side.
    smooth_size : int, pixels
        The sigma of the gaussian.
    peak_min_dist : int, pixels
        Min distance between nuclei.
    intensity_exp : int
        Exponent of the curve used to fit intensities on range [0,1]
    short_threshold_r : int, pixels
        Radius of neighborhood used to calculate a local
        average threshold.
    long_threshold_r : int, pixels
        Radius of neighborhood used to calculate a local
        average threshold
    cytoskeleton_threshold_r : int, pixels
        Radius of neighborhood used to calculate a local
        average threshold
    min_size_objects : float, micrometers^2
        Size beneath which no cells can exist.

    Returns
    -------
    (N, C) : list of (M,N) numpy arrays. Long dtype
        N is a labeled nucleus image. Where each label corresponds to an individual
        cell. 0 corresponds to the background. C is a labeled cytockeleton image. The
        labels correspond to the closest nucleus in N.
    '''

    # The parameters are adjusted based on an image with pixel scale of 0.48 micrometers/pixel
    zoomLev = 0.48 / pixel_scale

    N = np.sum(np.array(N), axis=2)
    N = ( (( N-np.amin(N)) / np.ptp(N)) )

    C = np.sum(np.array(C), axis=2)
    C = ( (( C-np.amin(C)) / np.ptp(C)) )

    # Step 1: smoothing intensity values and smoothing out peaks
    N = fftgauss(N, smooth_size, pad_type='edge')

    # Step 2: contrast enhancement by scaling intensities (from 0-1) on a curve
    ########  many other methods can be implemented for this step which could benefit the segmentation
    N = np.power(N/np.amax(N), intensity_exp)

    # Step 3: short and long range local avg threshold
    th_short = N > local_avg(N, short_threshold_r)
    th_long = N > local_avg(N, long_threshold_r)

    th_N = (th_short*th_long)
    del th_short, th_long

    # Step 4: remove small objects and smooth the binary image
    th_N = remove_small_objects(th_N, 20)
    th_N = remove_small_objects(th_N, min_size_objects * (zoomLev))

    th_N = smooth_binary(th_N, r=6, add_cond=0.5)
    th_N = smooth_binary(th_N, r=6, rem_cond=0.5)

    # Step 5: distance transform and generate markers from peaks
    distance = ndi.distance_transform_edt(th_N)
    peak_markers = corner_peaks(distance, min_distance=peak_min_dist, indices=False)
    peak_markers = ndi.label(peak_markers)[0]

    # Step 6: separate touching nuclei using the watershed markers
    label_N = watershed(th_N, peak_markers, mask=th_N)

    # Step 7: removing small regions after the watershed segmenation
    label_N = remove_small_objects(label_N, min_size_objects * (zoomLev))

    # Step 8: reassigning labels, so that they are continuously numbered
    label_N = relabel_sequential(label_N, offset=1)[0]

    # Step 14: local threshold of the cytoskeleton
    label_C = C > local_avg(C, cytoskeleton_threshold_r)
    label_C = smooth_binary(label_C, r=6, add_cond=0.5)
    label_C = smooth_binary(label_C, r=6, rem_cond=0.5)

    # Step 15: generate relabeled markers from the nuclei centroids
    new_markers = _gen_marks(label_N)

    # Step 16: watershed of cytoskeleton using new_markers
    label_C = watershed(label_C, new_markers, mask=label_C.astype(np.bool_))

    return [label_N, label_C]


def measure_fluor_cells(label_Nuc, label_Cyto, pixel_scale):
    '''
    Generates measurements for labeled Nucleus images and labeled Cytoskeleton images.

    Parameters
    ----------
    label_Nuc : (M,N) long dtype
        A labeled nucleus image. Where each label corresponds to an individual
        cell. 0 corresponds to the background.
    label_Cyto : (M,N) long dtype
        A labeled cytockeleton image. The labels correspond to the closest
        nucleus in N. 0 corresponds to the background.

    Returns
    -------
    Measurements : dataframe of measurements for each cell
        Cell_Index, Nuc_Area_um2, Nuc_Perimeter_um, Nuc_Area_Factor,
        Nuc_Major_L_um, Nuc_Minor_L_um, Nuc_eccentricity, Nuc_orientation,
        Nucleus_eq_diameter_um, Cyto_Area_um2, Cyto_Perimeter_um, Cyto_Area_Factor,
        Cyto_orientation, Cyto_Major_L_um, Cyto_Minor_L_um
    '''

    nuc_props = regionprops(label_Nuc, coordinates='rc')

    cell_index = 1

    col_names = all_measurements()

    prop_df = pd.DataFrame(columns = col_names)

    for i in range(len(nuc_props)):
        nucleus_A = nuc_props[i].area * (pixel_scale**2)
        nucleus_P = nuc_props[i].perimeter * pixel_scale
        nucleus_SF = 4 * np.pi * nucleus_A / (nucleus_P**2)

        cyto_props = regionprops((label_Cyto==i).astype(np.int), coordinates='rc')

        try:
            cyto_A = cyto_props[0].area * (pixel_scale**2)
            cyto_P = cyto_props[0].perimeter * pixel_scale
            cyto_SF = 4 * np.pi * cyto_A / (cyto_P**2)
            cyto_orientation = cyto_props[0].orientation
            cyto_Major_L_um = cyto_props[0].major_axis_length * pixel_scale
            cyto_Minor_L_um = cyto_props[0].minor_axis_length * pixel_scale

        except:
            cyto_A = None
            cyto_P = None
            cyto_SF = None
            cyto_orientation = None
            cyto_Major_L_um = None
            cyto_Minor_L_um = None

        prop_df = prop_df.append(pd.DataFrame(
                      {'Cell_Index': (cell_index,),
                       'Nuc_Area_um2': (nucleus_A,),
                       'Nuc_Perimeter_um': (nucleus_P,) ,
                       'Nuc_Area_Factor': (nucleus_SF,),
                       'Nuc_Major_L_um': (nuc_props[i].major_axis_length*pixel_scale,),
                       'Nuc_Minor_L_um': (nuc_props[i].minor_axis_length*pixel_scale,),
                       'Nuc_eccentricity': (nuc_props[i].eccentricity,),
                       'Nuc_orientation': (nuc_props[i].orientation,),
                       'Nucleus_eq_diameter_um': (nuc_props[i].equivalent_diameter*pixel_scale,),
                       'Cyto_Area_um2': (cyto_A,),
                       'Cyto_Perimeter_um': (cyto_P,),
                       'Cyto_Area_Factor': (cyto_SF,),
                       'Cyto_orientation': (cyto_orientation,),
                       'Cyto_Major_L_um': (cyto_Major_L_um,),
                       'Cyto_Minor_L_um': (cyto_Minor_L_um,)}))
        cell_index += 1

    # Reordering the columns
    prop_df = prop_df[col_names]


    return prop_df


def visualize_fluor_cells(L, A, thickness=1, bg_color='b', engine='matplotlib', figsize=(10,10)):
    '''
    Colors the original image with the segmented image. Also marks borders of
    segmentation on the original image so that borders can be evaluated.

    Parameters
    ----------
    L : (M,N) long dtype
        The labeled image that is a segmentation of A.
    A: (M,N) or (M,N,3) array
        The original image. Grayscale and color are supported.
        thickness : Thickness of the borders in pixels. Default is 1.
        color : Tuple of 3 uint8 RGB values.
    thickness: int
        Thickness of the yellow borders drawn on the im

    Returns
    -------
    (v1, v2) : tuple of (M,N,3) arrays uint8 dtype
        v1 is a colored original image. v2 is the original image with
        marked borders.
    '''

    A = np.array(A)

    if bg_color is 'b':
        bg_color=(0.1,0.1,0.5)
    elif bg_color is 'g':
        bg_color=(0.1,0.5,0.1)

    A = label2rgb(L, A, bg_label=0, bg_color=bg_color, alpha=0.1, image_alpha=1)

    A = np.interp(A, (0,1), (0,255)).astype(np.uint8)

    A = make_boundary_image(L, A, thickness=thickness)

    if engine == 'matplotlib':
        fig, ax = plt.subplots(1,1,figsize=figsize)
        ax.imshow(A)
        plt.show(fig)
    elif engine == 'PIL':
        A = Image.fromarray(A)
        A.show()
    elif engine == 'getimg':
        A = Image.fromarray(A)
        return A


def getparams(path):
    p = default_parameters('muscle_progenitor')

    with open(path, 'w+') as f:
        json.dump(p, f)


def segment(path_n, path_c, save_n, save_c, path_p, pxsize):
    print('Loading nuclei from `%s` and saving labels to `%s`.'%(path_n, save_n))
    nucleus_loader = dirload(path_n)
    print('Loading cytoskeletons from `%s` and saving labels to `%s`.'%(path_c, save_c))
    cyto_loader = dirload(path_c)
    print('Loading parameters from `%s`.'%path_p)
    with open(path_p, 'r') as f:
        p = json.load(f)

    for i, (N, C) in enumerate(zip(nucleus_loader, cyto_loader)):
        # Make sure that the image ids for nucleus and cytoskeleton are the same
        assert nucleus_loader.getname(i) == cyto_loader.getname(i)

        print('\nSegmenting Nucleus and Cytoskeleton with id: `%s`.'%nucleus_loader.getname(i))

        # Do segmentation. Pass all of the parameters in order.
        Label_N, Label_C = segment_fluor_cells(N, C, p['smooth_size'], p['intensity_exp'], p['short_threshold_r'], p['long_threshold_r'], p['min_size_objects'], p['peak_min_dist'], p['min_size_objects'], p['cytoskeleton_threshold_r'], pxsize)

        # Convert Images to PIL objects
        Label_N = visualize_fluor_cells(Label_N, N, thickness=2, bg_color='b', engine='getimg')
        Label_C = visualize_fluor_cells(Label_C, C, thickness=2, bg_color='g', engine='getimg')

        # Save Labeled Images to File
        path = os.path.join(save_n, nucleus_loader.getname(i) + '.png')
        Label_N.save(path)
        print('Saved labeled nuclei to `%s`.'%path)

        path = os.path.join(save_c, nucleus_loader.getname(i) + '.png')
        Label_C.save(path)
        print('Saved labeled cytoskeletons to `%s`.'%path)



def measure(path_n, path_c, save_m, path_p, pxsize):
        print('Loading nuclei from `%s`.'%path_n)
        nucleus_loader = dirload(path_n)
        print('Loading cytoskeletons from `%s`.'%path_c)
        cyto_loader = dirload(path_c)
        print('Loading parameters from `%s`.'%path_p)
        with open(path_p, 'r') as f:
            p = json.load(f)
        print('Saving measurements to `%s`.'%save_m)

        # Initialize Dataframe to save measurements
        col_names = ['Image_ID',] + all_measurements()
        M = pd.DataFrame(columns = col_names)

        for i, (N, C) in enumerate(zip(nucleus_loader, cyto_loader)):
            # Make sure that the image ids for nucleus and cytoskeleton are the same
            assert nucleus_loader.getname(i) == cyto_loader.getname(i)

            print('\nAnalyzing Nucleus and Cytoskeleton with id: `%s`.'%nucleus_loader.getname(i))

            # Do segmentation. Pass all of the parameters in order.
            Label_N, Label_C = segment_fluor_cells(N, C, p['smooth_size'], p['intensity_exp'], p['short_threshold_r'], p['long_threshold_r'], p['min_size_objects'], p['peak_min_dist'], p['min_size_objects'], p['cytoskeleton_threshold_r'], pxsize)

            # Get the measurements
            new_M = measure_fluor_cells(Label_N, Label_C, pxsize)
            new_M.insert(0, 'Image_ID', nucleus_loader.getname(i))

            # Append the measurements to the larger dataframe
            M = M.append(new_M)

        # Save Labeled Images to File
        M.to_csv(save_m)
        print('Saved measurements to `%s`.'%save_m)



from .grids import divergent
from .shapes import circle, donut, wheel, line, line_gen
from .weights import gauss, conical, drop

__all__ = ['divergent',
           'circle',
           'donut',
           'wheel',
           'gauss',
           'conical',
           'drop',
           'line',
           'line_gen']



import numpy as np
import matplotlib.pyplot as plt

def showimg(i, figsize=(8,8), cmap='gray', title=None, cb=False, vmin=None, vmax=None):
    if vmin is None:
        vmin = np.amin(i)
    if vmax is None:
        vmax = np.amax(i)
    fig, ax = plt.subplots(1,1,figsize=figsize)
    ax.imshow(i, cmap=cmap, vmin=vmin, vmax=vmax)
    ax.set_axis_off()
    if title:
        plt.title(title)
    if cb:
        plt.colorbar()
    plt.show()

def showcomp(imgs, figsize=(14,6), titles=None, vmin=None, vmax=None, horizontal=True):

    if not isinstance(vmin, list):
        vmin = [vmin,]*len(imgs)

    if not isinstance(vmax, list):
        vmax = [vmax,]*len(imgs)

    if horizontal:
        fig, axs = plt.subplots(1,len(imgs), figsize=figsize)
    else:
        fig, axs = plt.subplots(len(imgs),1, figsize=figsize)

    for i in range(len(imgs)):
        axs[i].imshow(imgs[i], cmap='gray', vmin=vmin[i], vmax=vmax[i])
        if titles is not None:
            if isinstance(titles, list):
                axs[i].set_title(titles[i])
        axs[i].set_axis_off()

    plt.show(fig)


def gplot(G, im=None, c=None, cmap='hot', figsize=(8,8), savefile=None, vmin=0, vmax=20, cb=False):
    fig, ax = plt.subplots(1, 1, figsize=figsize)

    if im is not None:
        ax.imshow(im)

    p = ax.scatter(G[:,1], G[:,0], c=c, cmap=cmap, vmin=vmin, vmax=vmax)
    ax.set_ylim(504,0)
    ax.set_xlim(0,504)
    ax.set_aspect('equal', 'box')
    # ax.set_facecolor((0.5,0.5,0.5))

    if cb:
        plt.colorbar(p)

    if savefile:
        fig.savefig(savefile)
        plt.close('all')
    else:
        plt.show(fig)



import numpy as np
from skimage.segmentation import find_boundaries
from skimage.morphology import binary_dilation, disk

def make_boundary_image(L, A, thickness=1, color=(255,255,85), rescale_hist=True):
    '''
    Marks borders of segmentation on the original image so that borders can
    be evaluated. Similar to skimage.segmentation.mark_boundaries. This function
    allows specifying a border thickness. This makes borders visible in high
    resolution images.

    Parameters
    ----------
    L : (M,N) array of dtype long
        The labeled image that is a segmentation of A.
    A : (M,N) or (M,N,3) array of any numerical dtype
        The original image. Grayscale and color are supported.
        thickness : Thickness of the borders in pixels. Default is 1.
        color : Tuple of 3 uint8 RGB values.

    Returns
    -------
    marked_image : ndarray
        The original image with marked borders, 3 color channels.
    '''

    if A.ndim == 2:
        A = np.stack((A,A,A), axis=2)

    if rescale_hist:
        A = np.interp(A, (np.amin(A), np.amax(A)), (0,255)).astype(np.uint8)
    else:
        A = A.astype(np.uint8)

    mask = find_boundaries(L)
    mask = binary_dilation(mask, selem=disk(thickness))

    R = A[:,:,0].copy()
    G = A[:,:,1].copy()
    B = A[:,:,2].copy()

    R[mask] = color[0]
    G[mask] = color[1]
    B[mask] = color[2]

    return np.stack((R,G,B), axis=2)



import numpy as np
from math import floor, ceil, cos, sin, pi

from ..common import circular_check
from .grids import divergent
from .shapes import circle


def gauss(sigma, theta=0, size=None, centered=True):
    '''
    Generates a 2d gaussian distribution. Usually used as a filter or kernel.

    Parameters
    ----------
    sigma : float
        The standard deviation of the gaussian. First value of tuple is the
        x st. dev., second value is the y st. dev.. If one value is given,
        x and y st. dev. are the same.
    size : tuple, optional
        The size of the output array that contains the object. Defaults to
        (round(4*sigma+1), round(4*sigma+1)).
    centered : boolean, optional
        If true, the center will be in the middle of the array
        at pixel (size[0]//2, size[1]//2). If false, the center will be
        at the origin pixel (0,0). Defaults to True.

    Returns
    -------
    gauss_distribution : ndarray
        A 2d gaussian distribution.
    '''

    if isinstance(sigma, list) or isinstance(sigma, tuple):
        sx, sy = sigma
    else:
        sx = sigma
        sy = sigma

    if size is None:
        r_x = 2*floor(sx)+1
        r_y = 2*floor(sy)+1

        coord = np.array([[-r_x, -r_y],
                          [-r_x, r_y],
                          [r_x, -r_y],
                          [r_x, r_y]
                         ])

        rotation = np.array([[np.cos(theta), np.sin(theta)],
                             [-np.sin(theta), np.cos(theta)]
                            ])

        coord = np.matmul(coord, rotation)
        size = (ceil(np.amax(coord[:,1]))-floor(np.amin(coord[:,1])), ceil(np.amax(coord[:,0]))-floor(np.amin(coord[:,0])))

    X, Y = divergent(size, centered)

    D = X**2 + Y**2

    a = cos(theta)**2 / (2 * sx**2) + sin(theta)**2 / (2 * sy**2)
    b = -sin(2*theta) / (4 * sx**2) + sin(2*theta) / (4 * sy**2)
    c = sin(theta)**2 / (2 * sx**2) + cos(theta)**2 / (2 * sy**2)

    return (1 / (2*pi*sx*sy)) * np.exp(-(a*X*X + 2*b*X*Y + c*Y*Y))


def conical(r, slope=1, size=None, centered=True):
    '''
    Generates a 2d array of z values for a cone in range [0,1]. The location of each value is
    tied to the x,y location of the pixel.

    Parameters
    ----------
    r : numeric
        The radius of the base of the cone
    slope : float
        The slope of the sides of the cone. Determines how quickly the cone approaches the
        maximum value of 1.
    size : tuple, optional
        The size of the output array that contains the object. Defaults to
        (round(4*sigma+1), round(4*sigma+1)).
    centered : boolean, optional
        If true, the center will be in the middle of the array
        at pixel (size[0]//2, size[1]//2). If false, the center will be
        at the origin pixel (0,0). Defaults to True.

    Returns
    -------
    cone : ndarray
        A cone described by a 2d array.
    '''

    size = circular_check(r, size)

    if centered:
        c = [i//2 for i in size]
    else:
        c = [0,0]

    K = circle(r, size, centered)

    shifts = np.argwhere(K) - c
    cone = np.zeros(size)

    for i, s in enumerate(shifts):
        cone[c[0]+s[0], c[1]+s[1]] = slope * (1 - (np.sqrt(s[0]**2+s[1]**2)/r))

    cone[cone<0] = 0

    return cone


def drop(r, threshold=None, size=None, centered=True):
    '''
    Generates a 2d array of z values for a drop shape in range [0,1]. The location of each
    z value is tied to the x,y location of the pixel.

    Parameters
    ----------
    r : numeric
        The radius of the base of the drop.
    threshold : float
        The relative values beneath which we determine the drop is equal to 0.
    size : tuple, optional
        The size of the output array that contains the object. Defaults to
        (round(4*sigma+1), round(4*sigma+1)).
    centered : boolean, optional
        If true, the center will be in the middle of the array
        at pixel (size[0]//2, size[1]//2). If false, the center will be
        at the origin pixel (0,0). Defaults to True.

    Returns
    -------
    drop : ndarray
        A drop described by a 2d array.
    '''

    size = circular_check(r, size)

    if centered:
        c = [i//2 for i in size]
    else:
        c = [0,0]

    K = circle(r, size, centered)
    K[c[0], c[1]] = 0

    shifts = np.argwhere(K) - c
    drop = np.zeros(size)
    for i, s in enumerate(shifts):
        drop[c[0]+s[0], c[1]+s[1]] = 1 / np.sqrt(s[0]**2+s[1]**2)

    drop[c[0], c[1]] = 1

    if threshold:
        drop[cone<threshold] = threshold

    return drop



import numpy as np
from numpy.fft import fftn, ifftn, fftshift, ifftshift

def maskfourier(image, mask, centered=True):
    '''
    Masks the fourier transform of an image using the given mask.

    Parameters
    ----------
    image : (M,N) array or (M,N,3) array
    mask : An (M,N) array
        Will be expanded to 3 channels if image is color.
    centered : boolean, optional
        Is the mask for a centered fourier transform?

    Returns
    -------
    (A, B) : tuple of ndarrays
        Two images generated from the masked fourier transforms. A contains
        an image reconstructed from values in the mask. B contains an image
        reconstructed from values outside the mask.

    Examples
    --------
    >>> import numpy as np
    >>> from imagemks.masking import maskfourier
    >>> image = np.ones((3,3))
    >>> mask_centered = np.array([[0,1,0],
                                  [1,1,1],
                                  [0,1,0]])

    >>> mask_origin = np.fft.ifftshift(mask_centered)
    >>> mask_origin
    array([[1, 1, 1],
           [1, 0, 0],
           [1, 0, 0]])

    The difference between centered masks and uncentered can be resolved by
    using np.fft.fftshift and np.ifftshift. By default all masks in structures
    are centered.

    >>> maskfourier(image, mask_centered, centered=True)
    (array([[1., 1., 1.],
            [1., 1., 1.],
            [1., 1., 1.]]),
     array([[0., 0., 0.],
            [0., 0., 0.],
            [0., 0., 0.]]))
    '''

    if image.ndim == 3:
        mask = np.stack((mask,mask,mask), axis=2)

    H_low = fftn(image, axes=(0,1))

    if centered:
        H_low = fftshift(H_low, axes=(0,1))

    H_high = H_low.copy()

    H_low[np.logical_not(mask)] = 0
    H_high[mask] = 0

    if centered:
        H_low = ifftshift(H_low)
        H_high = ifftshift(H_high)

    F_low = ifftn(H_low).real
    F_high = ifftn(H_high).real

    return (F_low, F_high)



import numpy as np

def divergent(size, centered=True):
    '''
    Creates two grids that specify the x and y values at each point in 2d space.

    Parameters
    ----------
    size : tuple
        The size of the output grids.
    centered : boolean, optional
        If true, the center will be in the middle of the array
        at pixel (size[0]//2, size[1]//2). If false, the center will be
        at the origin pixel (0,0). Defaults to True.

    Returns
    -------
    divergent_grid : tuple of ndarrays
        A grid that defines the x and y values at each point in 2d space. If the
        graph quadrants are recalled, a centered grid will produce an array with
        quadrants ordered as such:

        [ 2 | 1 ]

        [-------]

        [ 3 | 4 ]
    '''
    s0, s1 = size

    X, Y = np.meshgrid(np.arange(-s1//2 + s1%2, s1//2 + s1%2), np.arange(-s0//2 + s0%2, s0//2 + s0%2))

    if centered:
        return (X, Y)

    else:
        return (np.roll(X, -s1//2+s1%2, axis=1), np.roll(Y, -s0//2+s0%2, axis=0))



import numpy as np
from ._fftconvolve import fftconvolve2d
from ..structures import circle

def local_avg(img, rad, mask=None):
    '''
    Returns the local average of a neighborhood taking into account that the
    edges need to be treated differently. Averages are only calculated at the
    edges using values inside the image and the neighborhood.

    Parameters
    ----------
    img : (M,N) array
        An image.
    rad : numeric
        The radius of the neighborhood.
    mask : (M,N) binary array, optional
        A binary array that can be used to define what is outside the
        image.

    Returns
    -------
    local_averages : (M,N) array
        The local averages at all pixel locations.
    '''
    s = img.shape

    if mask is None:
        counts = np.ones(s)
    else:
        counts = mask.copy()

    pad_r = rad+1

    K = circle(rad, size=s)

    sums = fftconvolve2d(img, K, r=pad_r)
    norms = fftconvolve2d(counts, K, r=pad_r)

    return np.divide(sums, norms)



import numpy as np
from numpy.fft import fftn, ifftn, fftshift, ifftshift

def fftconvolve2d(h1, h2, r, pad_type=None, centered=True, **kwargs):
    '''
    This method adds to the functionality of scipy.signal.fftconvolve
    by providing padding options for 2D arrays.

    Parameters
    ----------
    h1 : (M,N) array, numeric
        The signal to be convolved, should be (M,N) and is usually the image.
    h2 : (M,N) array, numeric
        The signal to convolve with, should be (M,N) and is usually the filter.
        Will always be padded with zeros.
    r : int
        The padding width. Termed r since an int value will introduce a radius
        around the input. Different padding widths not supported.
    pad_type : string, optional
        The padding type to be used. For additional information see numpy.pad .
        Defaults to constant.
    centered : boolean, optional
        Needed since the filters in this package can be defined as centered
        or origin type.
    kwargs : varies
        See numpy.pad . Defaults to constant_values=0.

    Returns
    -------
    a : (M,N) array, float
        A convolved 2d signal in real space. Imaginary outputs not supported.
    '''

    assert(h1.ndim==2 and h2.ndim==2)
    assert(h1.shape==h2.shape)
    # Check that r is an int. Other pad widths not supported bc of return slicing.
    assert(isinstance(r,int))

    s0, s1 = h1.shape

    if pad_type:
        h1 = np.pad(h1, pad_width=r, mode=pad_type, **kwargs)
    else:
        h1 = np.pad(h1, pad_width=r, mode='constant', constant_values=0)

    if not centered:
        h2 = np.fft.fftshift(h2)

    h2 = np.pad(h2, pad_width=r, mode='constant', constant_values=0)

    # Performing the convolution of image with the kernel
    H1 = fftn(h1)
    H2 = fftn(h2).conj()
    conv = np.fft.fftshift(ifftn(H1*H2)).real

    return conv[r:s0+r,r:s1+r]



from . import filters
from . import masking
from . import structures
from . import visualization
from . import workflows

def get_version():
    """Get the version of the code from egg_info.
    Returns:
      the package version number
    """
    from pkg_resources import get_distribution, DistributionNotFound

    try:
        version = get_distribution(__name__.split('.')[0]).version # pylint: disable=no-member
    except DistributionNotFound: # pragma: no cover
        version = "unknown, try running `python setup.py egg_info`"

    return version

__version__ = get_version()

__all__ = ['__version__',
           'filters',
           'masking',
           'structures',
           'visualization',
           'workflows']



# Cell Workflow
from .fluorescentcells import segment_fluor_cells, measure_fluor_cells, visualize_fluor_cells, default_parameters
# Cell Program
from .fluorescentcells import getparams as cellgetparams, segment as cellsegment, measure as cellmeasure

__all__ = ['segment_fluor_cells',
           'measure_fluor_cells',
           'visualize_fluor_cells',
           'default_parameters',
           'cellgetparams',
           'cellsegment',
           'cellmeasure']



import sys
import os
from imagemks.workflows import cellgetparams, cellsegment, cellmeasure

def getoption(args, option):
    ind = [i for i, val in enumerate(args) if val == option]

    if len(ind) == 1:
        return args[ind[0]+1]
    else:
        raise ValueError('Multiple or none of required option (%s) found in command.'%option)


def cellanalysis_routine(args):
    subcommand = args[2]

    subcommands = [
        'info',
        'getparams',
        'segment',
        'measure',
    ]

    if subcommand == 'info':
        print("Available commands for program: " + ', '.join(subcommands))
    elif subcommand == 'getparams':
        save_p = getoption(args, '-sp')
        cellgetparams(save_p)
        print('Saved parameters to `%s`.'%save_p)
    elif subcommand == 'segment':
        path_n = getoption(args, '-fn')
        path_c = getoption(args, '-fc')
        save_n = getoption(args, '-sn')
        save_c = getoption(args, '-sc')
        path_p = getoption(args, '-fp')
        pxsize = float(getoption(args, '-z'))
        cellsegment(path_n, path_c, save_n, save_c, path_p, pxsize)
    elif subcommand == 'measure':
        path_n = getoption(args, '-fn')
        path_c = getoption(args, '-fc')
        save_m = getoption(args, '-sm')
        path_p = getoption(args, '-fp')
        pxsize = float(getoption(args, '-z'))
        cellmeasure(path_n, path_c, save_m, path_p, pxsize)
    else:
        print("Command not found. Run `cellanalysis info` for available commands.")



if __name__ == '__main__':

    commands = [
        'download',
        'info',
        'cellanalysis'
    ]

    if len(sys.argv) == 1:
        print("Available commands: " + ', '.join(commands))

    else:
        command = sys.argv[1]

        if command == 'info':
            print("Available commands for program: " + ', '.join(commands))

        elif command == 'cellanalysis':
            cellanalysis_routine(sys.argv)

        else:
            print("Command not found. Run `imagemks info` for available commands.")



from .borders import make_boundary_image
from .plotting import histvis
from .img_show import showimg, showcomp, gplot

__all__ = ['make_boundary_image',
           'histvis',
           'showimg',
           'showcomp',
           'gplot']



import numpy as np
from math import floor, ceil
from .grids import divergent


def circle(r, size=None, centered=True, dtype=np.bool_):
    '''
    Makes a circle with specified dtype. If bool or int, can be used as a mask.

    Parameters
    ----------
    r : numeric
        The radius of the circle.
    size : tuple, optional
        The size of the output array that contains the circle. Defaults to
        (round(2*r+1), round(2*r+1)).
    centered : boolean, optional
        If true, the circle will be centered in the middle of the array
        at pixel (size[0]//2, size[1]//2). If false, the circle will be centered
        at the origin pixel (0,0). Defaults to True.
    dtype : object, optional
        A valid numpy dtype. Defaults to boolean.

    Returns
    -------
    circle : ndarray
        A circle that obeys the equation :math:`x^2 + y^2 < r^2`
    '''

    if size is None:
        size = (round(2*r+1), round(2*r+1))

    X, Y = divergent(size, centered)

    return (X**2 + Y**2 <= r**2).astype(dtype)


def donut(r_outer, r_inner, size=None, centered=True, dtype=np.bool_):
    '''
    Makes a 2d donut with specified dtype. If bool or int, can be used as a mask.

    Parameters
    ----------
    r_outer : numeric
        The radius of the outer border.
    r_inner : numeric
        The radius of the inner border.
    size : tuple, optional
        The size of the output array that contains the donut. Defaults to
        (round(2*r_outer+1), round(2*r_outer+1)).
    centered : boolean, optional
        If true, the donut will be centered in the middle of the array
        at pixel (size[0]//2, size[1]//2). If false, the donut will be centered
        at the origin pixel (0,0). Defaults to True.
    dtype : object, optional
        A valid numpy dtype. Defaults to boolean.

    Returns
    -------
    donut : ndarray
        A donut that obeys the equation :math:`r_inner^2 < x^2 + y^2 < r_outer^2`
    '''

    if size is None:
        size = (round(2*r_outer+1), round(2*r_outer+1))

    X, Y = divergent(size, centered)

    D = X**2 + Y**2

    return np.logical_and(D <= r_outer**2, D>=r_inner**2).astype(dtype)


def line(a, size, width=None, r=None, centered=True, dtype=np.bool_):
    X, Y = divergent(size, centered)

    if width:
        line = np.logical_and(np.sin(a)*X + np.cos(a)*Y < width, np.sin(a)*X + np.cos(a)*Y > -width)
    else:
        line = (np.sin(a)*X + np.cos(a)*Y) == 0

    if r:
        line = np.logical_and(X**2+Y**2 <= r, line)

    return line.astype(dtype)


def line_gen(n, size, width=None, r=None, start=0, centered=True, dtype=np.bool_):
    return (line(a, size=size, width=width, r=r) for a in np.linspace(start, np.pi-np.pi/n+start, n))


def wheel(n_quad, size, width=None, r=None, start=0, centered=True, dtype=np.bool_):
    '''
    Makes a 2d wheel with specified dtype. If bool or int, can be used as a mask.

    Parameters
    ----------
    n_quad : int
        The number of spokes per quadrant (graph quadrant).
    size : tuple, optional
        The size of the output array that contains the wheel.
    width : int
        The width of a spoke.
    r : numeric, optional
        The maximum length of a spoke. Optional.
    start : float, optional
        Offset of the first spoke from 0 in radians.
    centered : boolean, optional
        If true, the wheel will be centered in the middle of the array
        at pixel (size[0]//2, size[1]//2). If false, the wheel will be centered
        at the origin pixel (0,0).
    dtype : object, optional
        A valid numpy dtype.

    Returns
    -------
    wheel : ndarray
        A wheel that is composed of lines (called spokes) that are evenly rotated
        around the center pixel.
    '''

    wheel = np.zeros(size)

    for a in np.linspace(start, np.pi-np.pi/(2*n_quad)+start, 2*n_quad):
        mask = line(a, size=size, width=width, start=start)
        wheel = np.logical_or(wheel, mask)

    if r:
        wheel = np.logical_and(X**2+Y**2 <= r, wheel)

    return wheel.astype(dtype)



def circular_check(r, size):
    if size is None:
        return (round(2*r+1), round(2*r+1))
    else:
        assert isinstance(size, list) or isinstance(size, tuple)
        return size


def oriented_check(r, size):
    if size is None:
        return (round(2*r+1), round(2*r+1))



from distutils.core import setup
from setuptools import find_packages

setup(
    name='ImageMKS',
    version='0.1.0',
    packages=find_packages(),
    license='The MIT License (MIT)',
    long_description=open('README.md').read(),
    author='Sven Voigt',
    author_email='svenpvoigt@gmail.com',
    url='http://pypi.python.org/pypi/ImageMKS/',
    description='Sharing segmentation frameworks.',
    scripts=['bin/imagemks', 'bin/cellanalysis'],
    install_requires=[
        'scipy',
        'numpy',
        'matplotlib',
        'pandas',
        'scikit-image',
        'scikit-learn'
    ],
)



from sklearn.decomposition import PCA
from sklearn.cluster import KMeans, AgglomerativeClustering

def tofloat(A):
    return np.interp(A, (np.amin(A), np.amax(A)), (0,1)).astype(np.float32)


def toint(A):
    return np.interp(A, (np.amin(A), np.amax(A)), (0,255)).astype(np.uint8)


def image_coords(shape, centered=False, aslist=False):
    s0, s1 = shape

    if centered:
        X, Y = np.meshgrid(np.arange(-s1//2+s1%2, s1//2+s1%2), np.arange(-s0//2+s0%2, s0//2+s0%2))
    else:
        X, Y = np.meshgrid(np.arange(B0.shape[1]), np.arange(B0.shape[0]))

    if aslist:
        return [(x,y) for x,y in zip(X.ravel(), Y.ravel())]
    else:
        return (X, Y)


def image_plane(A):
    coords = image_coords(A.shape, aslist=True)
    lr = LinearRegression()
    lr.fit(coords, A.reshape(-1,1))

    return lr.predict(coords).reshape(A.shape)


def image_clusters(A, n=3):
    km = KMeans(n_clusters=n)
    km.fit(A.reshape(-1,1))
    return km.labels_.reshape(A.shape)


def image_inrange(A, minp, maxp):
    minp = np.percentile(A, minp)
    maxp = np.percentile(A, maxp)
    return rescale_intensity(A, in_range=(minp, maxp))


def locations(shape, r, step=1):
    sy, sx = shape
    X, Y = np.meshgrid(np.arange(r, sx-r, step), np.arange(r, sy-r, step))
    return [i for i in zip(Y.ravel(), X.ravel())]


def divide_image(img, r, step=1, max_n=None, start_n=0):
    shape = (img.shape[0], img.shape[1])
    locs = locations(shape, r, step)[start_n:]

    if img.ndim == 2:
        A = np.zeros((len(locs), (2*r+1)**2))
        for n, loc in enumerate(locs):
            j, i = loc
            A[n,:] = img[(j-r):(j+r+1), (i-r):(i+r+1)].ravel()

            if max_n:
                if n > max_n:
                    break
    else:
        A = np.zeros((len(locs), 3*(2*r+1)**2))
        for n, loc in enumerate(locs):
            j, i = loc
            A[n,:] = img[(j-r):(j+r+1), (i-r):(i+r+1), :].ravel()

            if max_n:
                if n > max_n:
                    break

    return A


def cluster_neighborhoods(A, r, n_c=None, kmeans=None, pca=None):
    if pca is None:
        pca = PCA()
    kmeans = KMeans(n_clusters=n_c)
    new_size = tuple(i-2*r for i in A.shape)

    X = divide_image(A, r, 1)

    pca.fit(X)
    X = pca.transform(X)

    kmeans.fit(X)
    S = kmeans.predict(X)

    S = S.reshape(new_size)

    A = A[r:(A.shape[0]-r),r:(A.shape[1]-r)]
    A = np.interp(A, (np.amin(A), np.amax(A)), (0,1))

    return (S, A)



import numpy as np
import matplotlib.pyplot as plt
from matplotlib import rcParams

def histvis(a, nbins, col1='blue', col2='cyan', label='x', skip_ticks=1, figsize=(10,6), savepath=None):
    hist, bins = np.histogram(a.ravel(), bins=nbins)

    colors = [col1 if i%2==0 else col2 for i in range(nbins)]

    fig, ax = plt.subplots(1,1,figsize=figsize)

    rcParams['axes.labelpad'] = 15
    rcParams['figure.autolayout'] = 1

    ax.bar(bins[:-1], hist, width=(bins[1]-bins[0]), color=colors, align='edge')
    ax.set_xlabel(label, fontsize=22)
    ax.set_ylabel('Count', fontsize=22)
    ax.set_xticks(ticks=bins[::skip_ticks])
    ax.set_xticklabels(labels=np.around(bins[::skip_ticks], 2))

    for tick in ax.xaxis.get_major_ticks():
        tick.label.set_fontsize(18)
    for tick in ax.yaxis.get_major_ticks():
        tick.label.set_fontsize(18)

    if savepath:
        fig.savefig(savepath, dpi=300)

    plt.show(fig)
